// SoilLayers.inc
// by John Siino  (02/26/2007)

#include "dataVector.inc"
#define Sand  1
#define Clay  2
#define CPhi  3

class SoilLayer
{

// members
int numLayers;

enum property
   {
	DPTH,       //  1 = Depth to top of layer (enum starts with 0)
	THK,        //  2 = Layer Thickness
	TYP,        //  3 = Type (Sand = 1.0, Clay = 2.0, C-Phi = 3.0)
	DUW,        //  4 = Dry Unit Wt
	SUW,        //  5 = Saturated Unit Wt
	GWT,        //  6 = Ground Water Table (bool = 1 or 0, true or false)
	PHI,        //  7 = Friction Angle
	CUU,        //  8 = Undrained Strength
	E50,        //  9 = E50 at Reference Pressure
	REF,        // 10 = Reference Pressure
	BOT         // 11 = BOTTOM LAYER (insert other properties above this one)
				// use this property to tell program that this is the bottom 
				// layer 0.0 = not bottom layer, 1.0 = bottom layer 
   }

dataVector Layer;

declare SoilLayer;
declare _SoilLayer();
declare AddLayer();
declare SetProp( int n, property j, double value );
declare GetProp( int n, property j ), double;
declare GetNumLayers(), int;
declare PrintLayer();

}

SoilLayer::SoilLayer()
	{
		Layer.ReSize( BOT );
		numLayers = 1;
		return;
	}

SoilLayer::_SoilLayer()
	{
		Layer._dataVector();
		numLayers = 0;
		return;
	}

SoilLayer::GetNumLayers(), int
	{
		return numLayers;
	}

SoilLayer::AddLayer()
	{
		Layer.ReSize( (BOT) * (1 + numLayers) );
		numLayers++;
		return;
	}

SoilLayer::SetProp( int n, property j, double value )
	{
		Layer.Set( (n-1) + j-1, value );
		return;
	}

SoilLayer::GetProp( int n, property j), double
	{
		return Layer.Get( (n-1) + j-1);
	}

SoilLayer::PrintLayer()
	{
		// string title, string subtitle, string linetext, int numdecimals, int spacing, unsigned int printflag );
		Layer.PrintVector("","","",2,2,0);
	}
