// dataVector.inc
// dataVector class  version 0.7
// by John Siino  date 02/26/2007

#typedef uint unsigned int
#define mlength 8          // mlength = 8 for double, 4 for int and float

declare import,GlobalAlloc(flags as uint,size as int),uint;
              // flags = 0x00 Fixed Memory, = 0x40 initializes memory to zero
              // size: (amount of storage required) times (double = 8, float, int = 4)
              // returns pointer/handle to memory location

declare import,GlobalReAlloc(handle as uint,newsize as int,flags as uint),uint;
              // flags = 0x40 initializes memory to zero
              // newsize: (amount of storage required) times (double = 8, float, int = 4)
              // returns pointer/handle to memory location

declare import,GlobalFree(handle as uint),int;
              // returns null if successful
              // pointer/handle to memory location

declare import,RtlMoveMemory(dest as int, source as int, size as int);
              // dest = pointer to the starting address of the move destination
              // source = pointer to the starting address of the block of memory to be moved
              // size: (amount of storage required) times (double = 8, float, int = 4)
              // returns pointer/handle to memory location

// ----------------------------------------------------------------------
class dataVector                    // Class for Column Vector of doubles
{  
  declare dataVector();             // class constructor
  declare _dataVector();            // class destructor
  
  //member functions
  declare virtual Get(int pos), double;          // get value at position
  declare virtual Set(int pos, double value);    // set value at position
  
  declare virtual Ins(int pos, int value);       // insert value at position
  declare virtual ReSize(int size);              // resize vector
  declare virtual GetCount(),int;                // get number of elements
  
  declare ZeroAll();                             // reset all elements to zero
  declare RemoveAll();                           // reset array to zero elements (none)
  
  declare virtual Write(string title, string subtitle, string linetext, uint printflag);  // utility to write vectors

  declare virtual Copy(dataVector B), uint;         // copy  A = B
  declare virtual Copy2(dataVector B), uint;        // copy  B = A
  declare virtual Swap(dataVector B), uint;         // swap  A <-> B

  declare virtual PrintVector( string title, string subtitle, string linetext, int numdecimals, int spacing, unsigned int printflag );

//member variables
  int m_mem;                    // pointer to vector's memory address
  int m_GrowBy;                 // amount to grow memory by
  int m_numItems;               // number of elements in the vector
  int m_allocated;              // size of memory allocated
}
// ----------------------------------------------------------------------
dataVector :: dataVector()
{
  m_GrowBy = 10;
  m_mem = GlobalAlloc(0x40, m_GrowBy * mlength); 
  m_numItems = 0;
  m_allocated = 10;
  return;
}
// ----------------------------------------------------------------------
dataVector :: _dataVector()
{
  //clean up our object
  if(m_mem)
  {
    GlobalFree(m_mem);
    m_mem = NULL;
  }
  m_numItems = 0;
  m_allocated = 0;
  return;
}
// ----------------------------------------------------------------------
dataVector :: Get(int pos), double
{
  if pos < m_numItems
  {
    return *(double)(m_mem + (pos * mlength));
  }
  return 0;
}
// ----------------------------------------------------------------------
dataVector :: Set(int pos, double value)
{
  if(pos < m_allocated)
  {
    *(double)(m_mem + (pos * mlength)) = value;
    if((pos+1) > m_numItems)
    {
      m_numItems = pos+1;
    }
  }
return;
}
// ----------------------------------------------------------------------
dataVector :: Ins(int pos, double value)
{
  unsigned int temp;
  int total;
  if( (pos + 1) > m_allocated)
  {
    //calculate how many blocks to allocate
    total = (pos+1) - m_allocated;
    total = (total & m_GrowBy) + 1;
    total *= m_GrowBy;
    
    //Grow the array
    temp = GlobalRealloc(m_mem, (m_allocated + total) * mlength, 0x40);
    if(temp)
    {
      m_mem = temp;
      if ( (pos+1) > m_numItems)     // correct m_numItems
         {
          m_numItems = pos+1; 
          }
    }
    else
    {
      //could not reallocate so copy
      temp = GlobalAlloc(0x40,(m_allocated + total) * mlength);
      RtlMoveMemory(temp, m_mem, m_allocated * mlength);
      GlobalFree( m_mem );
      m_mem = temp;
    }
    m_allocated += m_GrowBy;
  }
  Set(pos, value);
return;
}
// ----------------------------------------------------------------------
dataVector :: ReSize(int size)
{
  uint temp;
  if(size > m_allocated)
  {
    temp = GlobalRealloc( m_mem, (size - m_allocated) * mlength ,0x40);
    if(temp)
    {
      m_mem = temp;
    }
    else
    {
      temp = GlobalAlloc(0x40,(size - m_allocated) * mlength);
      RtlMoveMemory(temp, m_mem, m_allocated * mlength);
      GlobalFree(m_mem);
      m_mem = temp;

    }
    m_allocated = size;
  }
  if ( size > m_numItems)     // correct m_numItems
     {
      m_numItems = size; 
      }
return;
}
// ----------------------------------------------------------------------
dataVector :: GetCount(), int
{
  return m_numItems;
}
// ----------------------------------------------------------------------
dataVector :: ZeroAll()
{
  for int x = 0; x < GetCount(); x++
    {
     *(double)(m_mem + (x * mlength)) = 0.0;
    }
return;
}
// ----------------------------------------------------------------------
dataVector :: RemoveAll()
{
  if(m_allocated)
  {
    GlobalFree(m_mem);
    m_allocated = 10;
    m_numItems = 0;
    m_mem = GlobalAlloc(0x40,m_GrowBy * mlength);
  }
return;
}
// ----------------------------------------------------------------------
dataVector :: Write(string title, string subtitle, string linetext, uint printflag)
{
  writeln(title);
  writeln(subtitle);
  for int x = 0; x < GetCount()-1; x++
    {
      select (printflag)
        {
        case 0:
            writeln( NumToStr( Get(x), 2 ) + linetext );
        case 1:
          writeln( linetext + NumToStr( Get(x), 2 ) + "\n" );
        case 2:
          writeln( linetext + NumToStr(x) + " = " + NumToStr( Get(x), 2 ) + "\n" );
        default:
          writeln( NumToStr( Get(x), 4 ) + ", " );
        }
    }
      select (printflag)
        {
        case 0:
            writeln( NumToStr( Get(x), 2 ) + linetext );
        case 1:
          writeln( linetext + NumToStr( Get(x), 2 ) + "\n" );
        case 2:
          writeln( linetext + NumToStr(x) + " = " + NumToStr( Get(x), 2 ) + "\n" );
        default:
          writeln( NumToStr( Get(x), 4 ) );
        }
return;
}
// ----------------------------------------------------------------------
dataVector :: Copy(dataVector B), uint               // copy  A = B
{
  uint flag = 0;
  if (B.GetCount() != GetCount())
    {
      print("ERROR!  Vectors not the same dimension.");
    }
  else
    {
      for int x = 0; x < GetCount(); x++
        {
         *(double)(m_mem + (x * mlength)) = B.Get(x);
        }
      flag = 1;
    }
return flag;
}
// ----------------------------------------------------------------------
dataVector :: Copy2(dataVector B), uint               // copy  B = A
{
  uint flag = 0;
  if (B.GetCount() != GetCount())
    {
      print("ERROR!  Vectors not the same dimension.");
    }
  else
    {
      for int x = 0; x < GetCount(); x++
        {
         B.Set(x, *(double)(m_mem + (x * mlength)));
        }
      flag = 1;
    }
return flag;
}
// ----------------------------------------------------------------------
dataVector :: Swap(dataVector B), uint               // swap  A <-> B
{
  uint flag = 0, temp;
  if (B.GetCount() != GetCount())
    {
      print("ERROR!  Vectors not the same dimension.");
    }
  else
    {
      temp = B.m_mem;
      B.m_mem = m_mem;
      m_mem = temp;
      flag = 1;
    }
return flag;
}

// ----------------------------------------------------------------------
dataVector :: PrintVector( string title, string subtitle, string linetext, int numdecimals, int spacing, unsigned int printflag )
{
  int maxsize = 0;
  string value;
  for int x = 0; x < GetCount(); x++
    {
      maxsize = MaxInt( maxsize, sizeof( NumToStr( Get(x), numdecimals ) ) );
    }
  maxsize = maxsize + spacing;
  
  writeln(title);
  writeln(subtitle);

  for x = 0; x < GetCount()-1; x++
    {
      value = NumToStr( Get(x), numdecimals );
      select (printflag)
        {
        case 0:
          writeln( space$( maxsize - sizeof(value) ) + value + linetext );
        case 1:
          writeln( linetext + space$( maxsize - sizeof(value) ) + value + "\n" );
        case 2:
          writeln( linetext + NumToStr(x) + " = " + space$( maxsize - sizeof(value) ) + value + "\n" );
        default:
          writeln( value + ", " );
        }
    }
    value = NumToStr( Get(x), numdecimals );
    select (printflag)
      {
      case 0:
        writeln( space$( maxsize - sizeof(value) ) + value + linetext );
      case 1:
        writeln( linetext + space$( maxsize - sizeof(value) ) + value + "\n" );
      case 2:
        writeln( linetext + NumToStr(x) + " = " + space$( maxsize - sizeof(value) ) + value + "\n" );
      default:
        writeln( value );
      }
return;
}
// ----------------------------------------------------------------------
sub MaxInt( int a, int b ), int         // generic function for determining Maximum Interger
{
  if ( b > a )
    {
    int maxvalue = b;  
    }
  else
    {
    maxvalue = a;  
    }

  return maxvalue;
}

sub MaxDbl( double a, double b ), double         // generic function for determining Maximum Double
{
  if ( b > a )
    {
    double maxvalue = b;  
    }
  else
    {
    maxvalue = a;  
    }

  return maxvalue;
}
// ----------------------------------------------------------------------
